@@ -1,4 +1,4 @@
-// test.h
+// equal.h
 
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 
 #include <fst/fst.h>
 
+
 namespace fst {
 
 // Tests if two Fsts have the same states and arcs in the same order.
@@ -93,7 +94,7 @@ bool Equal(const Fst<Arc> &fst1, const Fst<Arc> &fst2, float delta = kDelta) {
                 << ", weight2 = " << arc2.weight;
         return false;
       } else  if (arc1.nextstate != arc2.nextstate) {
-        VLOG(1) << "Equal: mismatched input label:"
+        VLOG(1) << "Equal: mismatched next states:"
                 << " state = " << s1
                 << ", arc = " << a
                 << ", nextstate1 = " << arc1.nextstate
@@ -104,10 +105,12 @@ bool Equal(const Fst<Arc> &fst1, const Fst<Arc> &fst2, float delta = kDelta) {
       aiter2.Next();
 
     }
-    // Sanity checks
-    CHECK_EQ(fst1.NumArcs(s1), fst2.NumArcs(s2));
-    CHECK_EQ(fst1.NumInputEpsilons(s1), fst2.NumInputEpsilons(s2));
-    CHECK_EQ(fst1.NumOutputEpsilons(s1), fst2.NumOutputEpsilons(s2));
+    // Sanity checks: should never fail
+    if (fst1.NumArcs(s1) != fst2.NumArcs(s2) ||
+        fst1.NumInputEpsilons(s1) != fst2.NumInputEpsilons(s2) ||
+        fst1.NumOutputEpsilons(s1) != fst2.NumOutputEpsilons(s2)) {
+      FSTERROR() << "Equal: inconsistent arc/epsilon counts";
+    }
 
     siter1.Next();
     siter2.Next();
