@@ -22,9 +22,9 @@
 #define FST_LIB_FACTOR_WEIGHT_H__
 
 #include <algorithm>
-#include <tr1/unordered_map>
-using std::tr1::unordered_map;
-#include <fst/slist.h>
+#include <unordered_map>
+using std::unordered_map;
+using std::unordered_multimap;
 #include <string>
 #include <utility>
 using std::pair; using std::make_pair;
@@ -34,6 +34,7 @@ using std::vector;
 #include <fst/cache.h>
 #include <fst/test-properties.h>
 
+
 namespace fst {
 
 const uint32 kFactorFinalWeights = 0x00000001;
@@ -153,11 +154,10 @@ class FactorWeightFstImpl
  public:
   using FstImpl<A>::SetType;
   using FstImpl<A>::SetProperties;
-  using FstImpl<A>::Properties;
   using FstImpl<A>::SetInputSymbols;
   using FstImpl<A>::SetOutputSymbols;
 
-  using CacheBaseImpl< CacheState<A> >::AddArc;
+  using CacheBaseImpl< CacheState<A> >::PushArc;
   using CacheBaseImpl< CacheState<A> >::HasStart;
   using CacheBaseImpl< CacheState<A> >::HasFinal;
   using CacheBaseImpl< CacheState<A> >::HasArcs;
@@ -261,6 +261,15 @@ class FactorWeightFstImpl
     return CacheImpl<A>::NumOutputEpsilons(s);
   }
 
+  uint64 Properties() const { return Properties(kFstProperties); }
+
+  // Set error if found; return FST impl properties.
+  uint64 Properties(uint64 mask) const {
+    if ((mask & kError) && fst_->Properties(kError, false))
+      SetProperties(kError, kError);
+    return FstImpl<Arc>::Properties(mask);
+  }
+
   void InitArcIterator(StateId s, ArcIteratorData<A> *data) {
     if (!HasArcs(s))
       Expand(s);
@@ -305,13 +314,13 @@ class FactorWeightFstImpl
         FactorIterator fit(w);
         if (!(mode_ & kFactorArcWeights) || fit.Done()) {
           StateId d = FindState(Element(arc.nextstate, Weight::One()));
-          AddArc(s, Arc(arc.ilabel, arc.olabel, w, d));
+          PushArc(s, Arc(arc.ilabel, arc.olabel, w, d));
         } else {
           for (; !fit.Done(); fit.Next()) {
             const pair<Weight, Weight> &p = fit.Value();
             StateId d = FindState(Element(arc.nextstate,
                                           p.second.Quantize(delta_)));
-            AddArc(s, Arc(arc.ilabel, arc.olabel, p.first, d));
+            PushArc(s, Arc(arc.ilabel, arc.olabel, p.first, d));
           }
         }
       }
@@ -329,7 +338,7 @@ class FactorWeightFstImpl
         const pair<Weight, Weight> &p = fit.Value();
         StateId d = FindState(Element(kNoStateId,
                                       p.second.Quantize(delta_)));
-        AddArc(s, Arc(final_ilabel_, final_olabel_, p.first, d));
+        PushArc(s, Arc(final_ilabel_, final_olabel_, p.first, d));
       }
     }
     SetArcs(s);
@@ -395,7 +404,8 @@ class FactorWeightFst : public ImplToFst< FactorWeightFstImpl<A, F> > {
   typedef A Arc;
   typedef typename A::Weight Weight;
   typedef typename A::StateId StateId;
-  typedef CacheState<A> State;
+  typedef DefaultCacheStore<A> Store;
+  typedef typename Store::State State;
   typedef FactorWeightFstImpl<A, F> Impl;
 
   FactorWeightFst(const Fst<A> &fst)
