@@ -21,8 +21,9 @@
 #ifndef FST_LIB_RELABEL_H__
 #define FST_LIB_RELABEL_H__
 
-#include <tr1/unordered_map>
-using std::tr1::unordered_map;
+#include <unordered_map>
+using std::unordered_map;
+using std::unordered_multimap;
 #include <string>
 #include <utility>
 using std::pair; using std::make_pair;
@@ -32,6 +33,11 @@ using std::vector;
 #include <fst/cache.h>
 #include <fst/test-properties.h>
 
+
+#include <unordered_map>
+using std::unordered_map;
+using std::unordered_multimap;
+
 namespace fst {
 
 //
@@ -76,11 +82,27 @@ void Relabel(
       // only relabel if relabel pair defined
       typename unordered_map<Label, Label>::iterator it =
         input_map.find(arc.ilabel);
-      if (it != input_map.end()) {arc.ilabel = it->second; }
+      if (it != input_map.end()) {
+        if (it->second == kNoLabel) {
+          FSTERROR() << "Input symbol id " << arc.ilabel
+                     << " missing from target vocabulary";
+          fst->SetProperties(kError, kError);
+          return;
+        }
+        arc.ilabel = it->second;
+      }
 
       // relabel output
       it = output_map.find(arc.olabel);
-      if (it != output_map.end()) { arc.olabel = it->second; }
+      if (it != output_map.end()) {
+        if (it->second == kNoLabel) {
+          FSTERROR() << "Output symbol id " << arc.olabel
+                     << " missing from target vocabulary";
+          fst->SetProperties(kError, kError);
+          return;
+        }
+        arc.olabel = it->second;
+      }
 
       aiter.SetValue(arc);
     }
@@ -89,7 +111,6 @@ void Relabel(
   fst->SetProperties(RelabelProperties(props), kFstProperties);
 }
 
-
 //
 // Relabels either the input labels or output labels. The old to
 // new labels mappings are specified using an input Symbol set.
@@ -127,10 +148,6 @@ void Relabel(MutableFst<A> *fst,
       string isymbol = syms_iter.Symbol();
       int isymbol_val = syms_iter.Value();
       int new_isymbol_val = new_isymbols->Find(isymbol);
-      if (new_isymbol_val == -1)
-        LOG(FATAL) << "Symbol not found in relabel_isymbols: ["
-                   << isymbol
-                   << "]";
       ipairs.push_back(make_pair(isymbol_val, new_isymbol_val));
     }
     if (attach_new_isymbols)
@@ -144,10 +161,6 @@ void Relabel(MutableFst<A> *fst,
       string osymbol = syms_iter.Symbol();
       int osymbol_val = syms_iter.Value();
       int new_osymbol_val = new_osymbols->Find(osymbol);
-      if (new_osymbol_val == -1)
-        LOG(FATAL) << "Symbol not found in relabel_osymbols: ["
-                   << osymbol
-                   << "]";
       opairs.push_back(make_pair(osymbol_val, new_osymbol_val));
     }
     if (attach_new_osymbols)
@@ -178,12 +191,11 @@ class RelabelFstImpl : public CacheImpl<A> {
  public:
   using FstImpl<A>::SetType;
   using FstImpl<A>::SetProperties;
-  using FstImpl<A>::Properties;
   using FstImpl<A>::WriteHeader;
   using FstImpl<A>::SetInputSymbols;
   using FstImpl<A>::SetOutputSymbols;
 
-  using CacheImpl<A>::AddArc;
+  using CacheImpl<A>::PushArc;
   using CacheImpl<A>::HasArcs;
   using CacheImpl<A>::HasFinal;
   using CacheImpl<A>::HasStart;
@@ -195,7 +207,9 @@ class RelabelFstImpl : public CacheImpl<A> {
   typedef typename A::Label   Label;
   typedef typename A::Weight  Weight;
   typedef typename A::StateId StateId;
-  typedef CacheState<A> State;
+  typedef DefaultCacheStore<A> Store;
+  typedef typename Store::State State;
+
 
   RelabelFstImpl(const Fst<A>& fst,
                  const vector<pair<Label, Label> >& ipairs,
@@ -312,6 +326,15 @@ class RelabelFstImpl : public CacheImpl<A> {
     return CacheImpl<A>::NumOutputEpsilons(s);
   }
 
+  uint64 Properties() const { return Properties(kFstProperties); }
+
+  // Set error if found; return FST impl properties.
+  uint64 Properties(uint64 mask) const {
+    if ((mask & kError) && fst_->Properties(kError, false))
+      SetProperties(kError, kError);
+    return FstImpl<Arc>::Properties(mask);
+  }
+
   void InitArcIterator(StateId s, ArcIteratorData<A>* data) {
     if (!HasArcs(s)) {
       Expand(s);
@@ -337,7 +360,7 @@ class RelabelFstImpl : public CacheImpl<A> {
         if (it != output_map_.end()) { arc.olabel = it->second; }
       }
 
-      AddArc(s, arc);
+      PushArc(s, arc);
     }
     SetArcs(s);
   }
@@ -371,7 +394,8 @@ class RelabelFst : public ImplToFst< RelabelFstImpl<A> > {
   typedef typename A::Label   Label;
   typedef typename A::Weight  Weight;
   typedef typename A::StateId StateId;
-  typedef CacheState<A> State;
+  typedef DefaultCacheStore<A> Store;
+  typedef typename Store::State State;
   typedef RelabelFstImpl<A> Impl;
 
   RelabelFst(const Fst<A>& fst,
