@@ -33,6 +33,7 @@ using std::vector;
 #include <fst/shortest-distance.h>
 #include <fst/test-properties.h>
 
+
 namespace fst {
 
 template <class Arc, class Queue, class ArcFilter>
@@ -85,10 +86,11 @@ void SingleShortestPath(const Fst<Arc> &ifst,
   ofst->SetInputSymbols(ifst.InputSymbols());
   ofst->SetOutputSymbols(ifst.OutputSymbols());
 
-  if (ifst.Start() == kNoStateId)
+  if (ifst.Start() == kNoStateId) {
+    if (ifst.Properties(kError, false)) ofst->SetProperties(kError, kError);
     return;
+  }
 
-  vector<Weight> rdistance;
   vector<bool> enqueued;
   vector<StateId> parent;
   vector<Arc> arc_parent;
@@ -100,18 +102,26 @@ void SingleShortestPath(const Fst<Arc> &ifst,
 
   distance->clear();
   state_queue->Clear();
-  if (opts.nshortest != 1)
-    LOG(FATAL) << "SingleShortestPath: for nshortest > 1, use ShortestPath"
+  if (opts.nshortest != 1) {
+    FSTERROR() << "SingleShortestPath: for nshortest > 1, use ShortestPath"
                << " instead";
+    ofst->SetProperties(kError, kError);
+    return;
+  }
   if (opts.weight_threshold != Weight::Zero() ||
-      opts.state_threshold != kNoStateId)
-    LOG(FATAL) <<
+      opts.state_threshold != kNoStateId) {
+    FSTERROR() <<
         "SingleShortestPath: weight and state thresholds not applicable";
+    ofst->SetProperties(kError, kError);
+    return;
+  }
   if ((Weight::Properties() & (kPath | kRightSemiring))
-       != (kPath | kRightSemiring))
-      LOG(FATAL) << "SingleShortestPath: Weight needs to have the path"
-                 << " property and be right distributive: " << Weight::Type();
-
+      != (kPath | kRightSemiring)) {
+    FSTERROR() << "SingleShortestPath: Weight needs to have the path"
+               << " property and be right distributive: " << Weight::Type();
+    ofst->SetProperties(kError, kError);
+    return;
+  }
   while (distance->size() < source) {
     distance->push_back(Weight::Zero());
     enqueued.push_back(false);
@@ -135,6 +145,10 @@ void SingleShortestPath(const Fst<Arc> &ifst,
         f_distance = Plus(f_distance, w);
         f_parent = s;
       }
+      if (!f_distance.Member()) {
+        ofst->SetProperties(kError, kError);
+        return;
+      }
       if (opts.first_path)
         break;
     }
@@ -153,6 +167,10 @@ void SingleShortestPath(const Fst<Arc> &ifst,
       Weight w = Times(sd, arc.weight);
       if (nd != Plus(nd, w)) {
         nd = Plus(nd, w);
+        if (!nd.Member()) {
+          ofst->SetProperties(kError, kError);
+          return;
+        }
         parent[arc.nextstate] = s;
         arc_parent[arc.nextstate] = arc;
         if (!enqueued[arc.nextstate]) {
@@ -164,8 +182,6 @@ void SingleShortestPath(const Fst<Arc> &ifst,
       }
     }
   }
-  (*distance)[source] = Weight::One();
-  parent[source] = kNoStateId;
 
   StateId s_p = kNoStateId, d_p = kNoStateId;
   for (StateId s = f_parent, d = kNoStateId;
@@ -181,6 +197,7 @@ void SingleShortestPath(const Fst<Arc> &ifst,
     }
   }
   ofst->SetStart(s_p);
+  if (ifst.Properties(kError, false)) ofst->SetProperties(kError, kError);
   ofst->SetProperties(
       ShortestPathProperties(ofst->Properties(kFstProperties, false)),
       kFstProperties);
@@ -241,12 +258,16 @@ class ShortestPathCompare {
 // natural semiring order. The single path that can be read from the
 // ith of at most n transitions leaving the initial state of 'ofst' is
 // the ith shortest path. Disregarding the initial state and initial
-// transitions, The n-shortest paths, in fact, form a tree rooted at
+// transitions, the n-shortest paths, in fact, form a tree rooted at
 // the single final state.
 //
 // The weights need to be left and right distributive (kSemiring) and
 // have the path (kPath) property.
 //
+// Arc weights must satisfy the property that the sum of the weights of one or
+// more paths from some state S to T is never Zero(). In particular, arc weights
+// are never Zero().
+//
 // The algorithm is from Mohri and Riley, "An Efficient Algorithm for
 // the n-best-strings problem", ICSLP 2002. The algorithm relies on
 // the shortest-distance algorithm. There are some issues with the
@@ -269,10 +290,13 @@ void NShortestPath(const Fst<RevArc> &ifst,
   typedef typename RevArc::Weight RevWeight;
 
   if (n <= 0) return;
-  if ((Weight::Properties() & (kPath | kSemiring)) != (kPath | kSemiring))
-    LOG(FATAL) << "NShortestPath: Weight needs to have the "
+  if ((Weight::Properties() & (kPath | kSemiring)) != (kPath | kSemiring)) {
+    FSTERROR() << "NShortestPath: Weight needs to have the "
                  << "path property and be distributive: "
                  << Weight::Type();
+    ofst->SetProperties(kError, kError);
+    return;
+  }
   ofst->DeleteStates();
   ofst->SetInputSymbols(ifst.InputSymbols());
   ofst->SetOutputSymbols(ifst.OutputSymbols());
@@ -298,8 +322,10 @@ void NShortestPath(const Fst<RevArc> &ifst,
       distance.size() <= ifst.Start() ||
       distance[ifst.Start()] == Weight::Zero() ||
       less(weight_threshold, Weight::One()) ||
-      state_threshold == 0)
+      state_threshold == 0) {
+    if (ifst.Properties(kError, false)) ofst->SetProperties(kError, kError);
     return;
+  }
   ofst->SetStart(ofst->AddState());
   StateId final = ofst->AddState();
   ofst->SetFinal(final, Weight::One());
@@ -355,6 +381,7 @@ void NShortestPath(const Fst<RevArc> &ifst,
     }
   }
   Connect(ofst);
+  if (ifst.Properties(kError, false)) ofst->SetProperties(kError, kError);
   ofst->SetProperties(
       ShortestPathProperties(ofst->Properties(kFstProperties, false)),
       kFstProperties);
@@ -362,7 +389,7 @@ void NShortestPath(const Fst<RevArc> &ifst,
 
 
 // N-Shortest-path algorithm:  this version allow fine control
-// via the otpions argument. See below for a simpler interface.
+// via the options argument. See below for a simpler interface.
 //
 // 'ofst' contains the n-shortest paths in 'ifst'. 'distance' returns
 // the shortest distances from the source state to each state in
@@ -400,12 +427,19 @@ void ShortestPath(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
     return;
   }
   if (n <= 0) return;
-  if ((Weight::Properties() & (kPath | kSemiring)) != (kPath | kSemiring))
-    LOG(FATAL) << "ShortestPath: n-shortest: Weight needs to have the "
-                 << "path property and be distributive: "
-                 << Weight::Type();
+  if ((Weight::Properties() & (kPath | kSemiring)) != (kPath | kSemiring)) {
+    FSTERROR() << "ShortestPath: n-shortest: Weight needs to have the "
+               << "path property and be distributive: "
+               << Weight::Type();
+    ofst->SetProperties(kError, kError);
+    return;
+  }
   if (!opts.has_distance) {
     ShortestDistance(ifst, distance, opts);
+    if (distance->size() == 1 && !(*distance)[0].Member()) {
+      ofst->SetProperties(kError, kError);
+      return;
+    }
   }
   // Algorithm works on the reverse of 'fst' : 'rfst', 'distance' is
   // the distance to the final state in 'rfst', 'ofst' is built as the
@@ -428,7 +462,7 @@ void ShortestPath(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
   } else {
     vector<Weight> ddistance;
     DeterminizeFstOptions<ReverseArc> dopts(opts.delta);
-    DeterminizeFst<ReverseArc> dfst(rfst, *distance, &ddistance, dopts);
+    DeterminizeFst<ReverseArc> dfst(rfst, distance, &ddistance, dopts);
     NShortestPath(dfst, ofst, ddistance, n, opts.delta,
                   opts.weight_threshold, opts.state_threshold);
   }
