@@ -25,20 +25,127 @@
 #include <string>
 #include <vector>
 using std::vector;
+
 #include <fst/mutable-fst.h>
+#include <fst/fst-decl.h>  // For optional argument declarations
 #include <fst/test-properties.h>
 
+
 namespace fst {
 
-template <class A> class VectorFst;
+template <class A, class S> class VectorFst;
 template <class F, class G> void Cast(const F &, G *);
 
-
-// States and arcs implemented by STL vectors, templated on the
-// State definition. This does not manage the Fst properties.
-template <class State>
-class VectorFstBaseImpl : public FstImpl<typename State::Arc> {
+// Arcs (of type A) implemented by an STL vector per state. M specifies Arc
+// allocator (default declared in fst-decl.h).
+template <class A, class M /* = std::allocator<A> */>
+class VectorState {
  public:
+  typedef A Arc;
+  typedef typename A::Weight Weight;
+  typedef typename A::StateId StateId;
+  typedef M ArcAllocator;
+  typedef typename ArcAllocator::template
+  rebind<VectorState<A, M> >::other StateAllocator;
+
+  // Provide STL allocator for arcs
+  explicit VectorState(const ArcAllocator &alloc)
+      : final_(Weight::Zero()), niepsilons_(0), noepsilons_(0),
+        arcs_(alloc) {}
+
+  VectorState(const VectorState<A, M> &state, const ArcAllocator &alloc)
+      : final_(state.Final()),
+        niepsilons_(state.NumInputEpsilons()),
+        noepsilons_(state.NumOutputEpsilons()),
+        arcs_(state.arcs_.begin(), state.arcs_.end(), alloc) { }
+
+  void Reset() {
+    final_ = Weight::Zero();
+    niepsilons_ = 0;
+    noepsilons_ = 0;
+    arcs_.clear();
+  }
+
+  Weight Final() const { return final_; }
+  size_t NumInputEpsilons() const { return niepsilons_; }
+  size_t NumOutputEpsilons() const { return noepsilons_; }
+  size_t NumArcs() const { return arcs_.size(); }
+  const A &GetArc(size_t n) const { return arcs_[n]; }
+
+  const A *Arcs() const { return arcs_.size() > 0 ? &arcs_[0] : 0; }
+  A *MutableArcs() { return arcs_.size() > 0 ? &arcs_[0] : 0; }
+
+  void ReserveArcs(size_t n) { arcs_.reserve(n); }
+
+  void SetFinal(Weight final) { final_ =  final; }
+  void SetNumInputEpsilons(size_t n) { niepsilons_ = n; }
+  void SetNumOutputEpsilons(size_t n) { noepsilons_ = n; }
+
+  void AddArc(const A &arc) {
+    if (arc.ilabel == 0)
+      ++niepsilons_;
+    if (arc.olabel == 0)
+      ++noepsilons_;
+    arcs_.push_back(arc);
+  }
+
+  void SetArc(const A &arc, size_t n) {
+    if (arcs_[n].ilabel == 0)
+      --niepsilons_;
+    if (arcs_[n].olabel == 0)
+      --noepsilons_;
+    if (arc.ilabel == 0)
+      ++niepsilons_;
+    if (arc.olabel == 0)
+      ++noepsilons_;
+    arcs_[n] = arc;
+  }
+
+  void DeleteArcs() {
+    niepsilons_ = 0;
+    noepsilons_ = 0;
+    arcs_.clear();
+  }
+
+  void DeleteArcs(size_t n) {
+    for (size_t i = 0; i < n; ++i) {
+      if (arcs_.back().ilabel == 0)
+        --niepsilons_;
+      if (arcs_.back().olabel == 0)
+        --noepsilons_;
+      arcs_.pop_back();
+    }
+  }
+
+  // For state class allocation
+  void *operator new(size_t size, StateAllocator *alloc) {
+    return alloc->allocate(1);
+  }
+
+  // For state destruction and memory freeing
+  static void Destroy(VectorState<A, M> *state, StateAllocator *alloc) {
+    if (state) {
+      state->~VectorState<A, M>();
+      alloc->deallocate(state, 1);
+    }
+  }
+
+ private:
+  Weight final_;                  // Final weight
+  size_t niepsilons_;             // # of input epsilons
+  size_t noepsilons_;             // # of output epsilons
+  vector<A, ArcAllocator> arcs_;  // Arcs represenation
+
+  DISALLOW_COPY_AND_ASSIGN(VectorState);
+};
+
+
+// States are implemented by STL vectors, templated on the
+// State definition. This does not manage the Fst properties.
+template <class S>
+class VectorFstBaseImpl : public FstImpl<typename S::Arc> {
+ public:
+  typedef S State;
   typedef typename State::Arc Arc;
   typedef typename Arc::Weight Weight;
   typedef typename Arc::StateId StateId;
@@ -47,23 +154,31 @@ class VectorFstBaseImpl : public FstImpl<typename State::Arc> {
 
   ~VectorFstBaseImpl() {
     for (StateId s = 0; s < states_.size(); ++s)
-      delete states_[s];
+      State::Destroy(states_[s], &state_alloc_);
   }
 
   StateId Start() const { return start_; }
 
-  Weight Final(StateId s) const { return states_[s]->final; }
+  Weight Final(StateId s) const { return states_[s]->Final(); }
 
   StateId NumStates() const { return states_.size(); }
 
-  size_t NumArcs(StateId s) const { return states_[s]->arcs.size(); }
+  size_t NumArcs(StateId s) const { return states_[s]->NumArcs(); }
+
+  size_t NumInputEpsilons(StateId s) const {
+    return GetState(s)->NumInputEpsilons();
+  }
+
+  size_t NumOutputEpsilons(StateId s) const {
+    return GetState(s)->NumOutputEpsilons();
+  }
 
   void SetStart(StateId s) { start_ = s; }
 
-  void SetFinal(StateId s, Weight w) { states_[s]->final = w; }
+  void SetFinal(StateId s, Weight w) { states_[s]->SetFinal(w); }
 
   StateId AddState() {
-    states_.push_back(new State);
+    states_.push_back(new(&state_alloc_) State(arc_alloc_));
     return states_.size() - 1;
   }
 
@@ -73,7 +188,7 @@ class VectorFstBaseImpl : public FstImpl<typename State::Arc> {
   }
 
   void AddArc(StateId s, const Arc &arc) {
-    states_[s]->arcs.push_back(arc);
+    states_[s]->AddArc(arc);
   }
 
   void DeleteStates(const vector<StateId>& dstates) {
@@ -88,14 +203,16 @@ class VectorFstBaseImpl : public FstImpl<typename State::Arc> {
           states_[nstates] = states_[s];
         ++nstates;
       } else {
-        delete states_[s];
+        State::Destroy(states_[s], &state_alloc_);
       }
     }
     states_.resize(nstates);
     for (StateId s = 0; s < states_.size(); ++s) {
-      vector<Arc> &arcs = states_[s]->arcs;
+      Arc *arcs = states_[s]->MutableArcs();
       size_t narcs = 0;
-      for (size_t i = 0; i < arcs.size(); ++i) {
+      size_t nieps = states_[s]->NumInputEpsilons();
+      size_t noeps = states_[s]->NumOutputEpsilons();
+      for (size_t i = 0; i < states_[s]->NumArcs(); ++i) {
         StateId t = newid[arcs[i].nextstate];
         if (t != kNoStateId) {
           arcs[i].nextstate = t;
@@ -104,12 +221,14 @@ class VectorFstBaseImpl : public FstImpl<typename State::Arc> {
           ++narcs;
         } else {
           if (arcs[i].ilabel == 0)
-            --states_[s]->niepsilons;
+            --nieps;
           if (arcs[i].olabel == 0)
-            --states_[s]->noepsilons;
+            --noeps;
         }
       }
-      arcs.resize(narcs);
+      states_[s]->DeleteArcs(states_[s]->NumArcs() - narcs);
+      states_[s]->SetNumInputEpsilons(nieps);
+      states_[s]->SetNumOutputEpsilons(noeps);
     }
     if (Start() != kNoStateId)
       SetStart(newid[Start()]);
@@ -117,16 +236,16 @@ class VectorFstBaseImpl : public FstImpl<typename State::Arc> {
 
   void DeleteStates() {
     for (StateId s = 0; s < states_.size(); ++s)
-      delete states_[s];
+      State::Destroy(states_[s], &state_alloc_);
     states_.clear();
     SetStart(kNoStateId);
   }
 
   void DeleteArcs(StateId s, size_t n) {
-    states_[s]->arcs.resize(states_[s]->arcs.size() - n);
+    states_[s]->DeleteArcs(n);
   }
 
-  void DeleteArcs(StateId s) { states_[s]->arcs.clear(); }
+  void DeleteArcs(StateId s) { states_[s]->DeleteArcs(); }
 
   State *GetState(StateId s) { return states_[s]; }
 
@@ -136,7 +255,7 @@ class VectorFstBaseImpl : public FstImpl<typename State::Arc> {
 
   void ReserveStates(StateId n) { states_.reserve(n); }
 
-  void ReserveArcs(StateId s, size_t n) { states_[s]->arcs.reserve(n); }
+  void ReserveArcs(StateId s, size_t n) { states_[s]->ReserveArcs(n); }
 
   // Provide information needed for generic state iterator
   void InitStateIterator(StateIteratorData<Arc> *data) const {
@@ -147,53 +266,41 @@ class VectorFstBaseImpl : public FstImpl<typename State::Arc> {
   // Provide information needed for generic arc iterator
   void InitArcIterator(StateId s, ArcIteratorData<Arc> *data) const {
     data->base = 0;
-    data->narcs = states_[s]->arcs.size();
-    data->arcs = data->narcs > 0 ? &states_[s]->arcs[0] : 0;
+    data->narcs = states_[s]->NumArcs();
+    data->arcs = states_[s]->Arcs();
     data->ref_count = 0;
   }
 
  private:
   vector<State *> states_;      // States represenation.
   StateId start_;               // initial state
+  typename State::StateAllocator state_alloc_;  // for state allocation
+  typename State::ArcAllocator arc_alloc_;      // for arc allocation
 
   DISALLOW_COPY_AND_ASSIGN(VectorFstBaseImpl);
 };
 
-// Arcs implemented by an STL vector per state.
-template <class A>
-struct VectorState {
-  typedef A Arc;
-  typedef typename A::Weight Weight;
-  typedef typename A::StateId StateId;
-
-  VectorState() : final(Weight::Zero()), niepsilons(0), noepsilons(0) {}
-
-  Weight final;              // Final weight
-  vector<A> arcs;            // Arcs represenation
-  size_t niepsilons;         // # of input epsilons
-  size_t noepsilons;         // # of output epsilons
-};
-
 // This is a VectorFstBaseImpl container that holds VectorState's.  It
-// manages Fst properties and the # of input and output epsilons.
-template <class A>
-class VectorFstImpl : public VectorFstBaseImpl< VectorState<A> > {
+// manages Fst properties.
+template <class S>
+class VectorFstImpl : public VectorFstBaseImpl<S> {
  public:
+  typedef S State;
+  typedef typename S::Arc A;
   using FstImpl<A>::SetInputSymbols;
   using FstImpl<A>::SetOutputSymbols;
   using FstImpl<A>::SetType;
   using FstImpl<A>::SetProperties;
   using FstImpl<A>::Properties;
-  using FstImpl<A>::WriteHeader;
 
-  using VectorFstBaseImpl<VectorState<A> >::Start;
-  using VectorFstBaseImpl<VectorState<A> >::NumStates;
-  using VectorFstBaseImpl<VectorState<A> >::GetState;
-  using VectorFstBaseImpl<VectorState<A> >::ReserveArcs;
+  using VectorFstBaseImpl<S>::Start;
+  using VectorFstBaseImpl<S>::NumStates;
+  using VectorFstBaseImpl<S>::GetState;
+  using VectorFstBaseImpl<S>::ReserveArcs;
 
-  friend class MutableArcIterator< VectorFst<A> >;
+  friend class MutableArcIterator< VectorFst<A, S> >;
 
-  typedef VectorFstBaseImpl< VectorState<A> > BaseImpl;
+  typedef VectorFstBaseImpl<S> BaseImpl;
   typedef typename A::Weight Weight;
   typedef typename A::StateId StateId;
 
@@ -203,13 +310,7 @@ class VectorFstImpl : public VectorFstBaseImpl< VectorState<A> > {
   }
   explicit VectorFstImpl(const Fst<A> &fst);
 
-  static VectorFstImpl<A> *Read(istream &strm, const FstReadOptions &opts);
-
-  size_t NumInputEpsilons(StateId s) const { return GetState(s)->niepsilons; }
-
-  size_t NumOutputEpsilons(StateId s) const { return GetState(s)->noepsilons; }
-
-  bool Write(ostream &strm, const FstWriteOptions &opts) const;
+  static VectorFstImpl<S> *Read(istream &strm, const FstReadOptions &opts);
 
   void SetStart(StateId s) {
     BaseImpl::SetStart(s);
@@ -229,17 +330,10 @@ class VectorFstImpl : public VectorFstBaseImpl< VectorState<A> > {
   }
 
   void AddArc(StateId s, const A &arc) {
-    VectorState<A> *state = GetState(s);
-    if (arc.ilabel == 0) {
-      ++state->niepsilons;
-    }
-    if (arc.olabel == 0) {
-      ++state->noepsilons;
-    }
-
-    const A *parc = state->arcs.empty() ? 0 : &(state->arcs.back());
+    State *state = GetState(s);
+    const A *parc = state->NumArcs() == 0 ? 0 :
+        &(state->GetArc(state->NumArcs() - 1));
     SetProperties(AddArcProperties(Properties(), s, arc, parc));
-
     BaseImpl::AddArc(s, arc);
   }
 
@@ -250,33 +344,23 @@ class VectorFstImpl : public VectorFstBaseImpl< VectorState<A> > {
 
   void DeleteStates() {
     BaseImpl::DeleteStates();
-    SetProperties(DeleteAllStatesProperties(Properties(),
-                                            kStaticProperties));
+    SetProperties(DeleteAllStatesProperties(Properties(), kStaticProperties));
   }
 
   void DeleteArcs(StateId s, size_t n) {
-    const vector<A> &arcs = GetState(s)->arcs;
-    for (size_t i = 0; i < n; ++i) {
-      size_t j = arcs.size() - i - 1;
-      if (arcs[j].ilabel == 0)
-        --GetState(s)->niepsilons;
-      if (arcs[j].olabel == 0)
-        --GetState(s)->noepsilons;
-    }
     BaseImpl::DeleteArcs(s, n);
     SetProperties(DeleteArcsProperties(Properties()));
   }
 
   void DeleteArcs(StateId s) {
-    GetState(s)->niepsilons = 0;
-    GetState(s)->noepsilons = 0;
     BaseImpl::DeleteArcs(s);
     SetProperties(DeleteArcsProperties(Properties()));
   }
 
- private:
   // Properties always true of this Fst class
   static const uint64 kStaticProperties = kExpanded | kMutable;
+
+ private:
   // Current file format version
   static const int kFileVersion = 2;
   // Minimum file format version supported
@@ -285,18 +369,18 @@ class VectorFstImpl : public VectorFstBaseImpl< VectorState<A> > {
   DISALLOW_COPY_AND_ASSIGN(VectorFstImpl);
 };
 
-template <class A> const uint64 VectorFstImpl<A>::kStaticProperties;
-template <class A> const int VectorFstImpl<A>::kFileVersion;
-template <class A> const int VectorFstImpl<A>::kMinFileVersion;
+template <class S> const uint64 VectorFstImpl<S>::kStaticProperties;
+template <class S> const int VectorFstImpl<S>::kFileVersion;
+template <class S> const int VectorFstImpl<S>::kMinFileVersion;
 
-
-template <class A>
-VectorFstImpl<A>::VectorFstImpl(const Fst<A> &fst) {
+template <class S>
+VectorFstImpl<S>::VectorFstImpl(const Fst<A> &fst) {
   SetType("vector");
-  SetProperties(fst.Properties(kCopyProperties, false) | kStaticProperties);
   SetInputSymbols(fst.InputSymbols());
   SetOutputSymbols(fst.OutputSymbols());
   BaseImpl::SetStart(fst.Start());
+  if (fst.Properties(kExpanded, false))
+    BaseImpl::ReserveStates(CountStates(fst));
 
   for (StateIterator< Fst<A> > siter(fst);
        !siter.Done();
@@ -310,32 +394,37 @@ VectorFstImpl<A>::VectorFstImpl(const Fst<A> &fst) {
          aiter.Next()) {
       const A &arc = aiter.Value();
       BaseImpl::AddArc(s, arc);
-      if (arc.ilabel == 0)
-        ++GetState(s)->niepsilons;
-      if (arc.olabel == 0)
-        ++GetState(s)->noepsilons;
     }
   }
+  SetProperties(fst.Properties(kCopyProperties, false) | kStaticProperties);
 }
 
-template <class A>
-VectorFstImpl<A> *VectorFstImpl<A>::Read(istream &strm,
-                                         const FstReadOptions &opts) {
-  VectorFstImpl<A> *impl = new VectorFstImpl;
+template <class S>
+VectorFstImpl<S> *VectorFstImpl<S>::Read(istream &strm,
+                                                 const FstReadOptions &opts) {
+  VectorFstImpl<S> *impl = new VectorFstImpl;
   FstHeader hdr;
-  if (!impl->ReadHeader(strm, opts, kMinFileVersion, &hdr))
+  if (!impl->ReadHeader(strm, opts, kMinFileVersion, &hdr)) {
+    delete impl;
     return 0;
+  }
   impl->BaseImpl::SetStart(hdr.Start());
-  impl->ReserveStates(hdr.NumStates());
+  if (hdr.NumStates() != kNoStateId) {
+    impl->ReserveStates(hdr.NumStates());
+  }
 
-  for (StateId s = 0; s < hdr.NumStates(); ++s) {
+  StateId s = 0;
+  for (;hdr.NumStates() == kNoStateId || s < hdr.NumStates(); ++s) {
+    typename A::Weight final;
+    if (!final.Read(strm)) break;
     impl->BaseImpl::AddState();
-    VectorState<A> *state = impl->GetState(s);
-    state->final.Read(strm);
+    State *state = impl->GetState(s);
+    state->SetFinal(final);
     int64 narcs;
     ReadType(strm, &narcs);
     if (!strm) {
       LOG(ERROR) << "VectorFst::Read: read failed: " << opts.source;
+      delete impl;
       return 0;
     }
     impl->ReserveArcs(s, narcs);
@@ -344,18 +433,20 @@ VectorFstImpl<A> *VectorFstImpl<A>::Read(istream &strm,
       ReadType(strm, &arc.ilabel);
       ReadType(strm, &arc.olabel);
       arc.weight.Read(strm);
-       ReadType(strm, &arc.nextstate);
-       if (!strm) {
+      ReadType(strm, &arc.nextstate);
+      if (!strm) {
         LOG(ERROR) << "VectorFst::Read: read failed: " << opts.source;
+        delete impl;
         return 0;
       }
       impl->BaseImpl::AddArc(s, arc);
-      if (arc.ilabel == 0)
-        ++state->niepsilons;
-      if (arc.olabel == 0)
-        ++state->noepsilons;
     }
   }
+  if (hdr.NumStates() != kNoStateId && s != hdr.NumStates()) {
+    LOG(ERROR) << "VectorFst::Read: unexpected end of file: " << opts.source;
+    delete impl;
+    return 0;
+  }
   return impl;
 }
 
@@ -368,8 +459,8 @@ template <class W> class WeightFromString {
 // Generic case fails.
 template <class W> inline
 W WeightFromString<W>::operator()(const string &s) {
-  LOG(FATAL) << "VectorFst::Read: Obsolete file format";
-  return W();
+  FSTERROR() << "VectorFst::Read: Obsolete file format";
+  return W::NoWeight();
 }
 
 // TropicalWeight version.
@@ -388,88 +479,72 @@ LogWeight WeightFromString<LogWeight>::operator()(const string &s) {
   return LogWeight(f);
 }
 
-template <class A>
-bool VectorFstImpl<A>::Write(ostream &strm,
-                             const FstWriteOptions &opts) const {
-  FstHeader hdr;
-  hdr.SetStart(Start());
-  hdr.SetNumStates(NumStates());
-  WriteHeader(strm, opts, kFileVersion, &hdr);
-
-  for (StateId s = 0; s < NumStates(); ++s) {
-    const VectorState<A> *state = GetState(s);
-    state->final.Write(strm);
-    int64 narcs = state->arcs.size();
-    WriteType(strm, narcs);
-    for (size_t a = 0; a < narcs; ++a) {
-      const A &arc = state->arcs[a];
-      WriteType(strm, arc.ilabel);
-      WriteType(strm, arc.olabel);
-      arc.weight.Write(strm);
-      WriteType(strm, arc.nextstate);
-    }
-  }
-
-  strm.flush();
-  if (!strm) {
-    LOG(ERROR) << "VectorFst::Write: write failed: " << opts.source;
-    return false;
-  }
-  return true;
-}
-
-
 // Simple concrete, mutable FST. This class attaches interface to
 // implementation and handles reference counting, delegating most
 // methods to ImplToMutableFst. Supports additional operations:
-// ReserveStates and ReserveArcs (cf. STL vectors).
-template <class A>
-class VectorFst : public ImplToMutableFst< VectorFstImpl<A> > {
+// ReserveStates and ReserveArcs (cf. STL vectors). The second
+// optional template argument gives the State definition (default
+// declared in fst-decl.h).
+template <class A, class S /* = VectorState<A> */>
+class VectorFst : public ImplToMutableFst<VectorFstImpl<S> > {
  public:
-  friend class StateIterator< VectorFst<A> >;
-  friend class ArcIterator< VectorFst<A> >;
-  friend class MutableArcIterator< VectorFst<A> >;
-  template <class F, class G> void friend Cast(const F &, G *);
+  friend class StateIterator< VectorFst<A, S> >;
+  friend class ArcIterator< VectorFst<A, S> >;
+  friend class MutableArcIterator< VectorFst<A, S> >;
+  template <class F, class G> friend void Cast(const F &, G *);
 
   typedef A Arc;
   typedef typename A::StateId StateId;
-  typedef VectorFstImpl<A> Impl;
+  typedef S State;
+  typedef VectorFstImpl<State> Impl;
 
   VectorFst() : ImplToMutableFst<Impl>(new Impl) {}
 
   explicit VectorFst(const Fst<A> &fst)
       : ImplToMutableFst<Impl>(new Impl(fst)) {}
 
-  VectorFst(const VectorFst<A> &fst) : ImplToMutableFst<Impl>(fst) {}
+  VectorFst(const VectorFst<A, S> &fst) : ImplToMutableFst<Impl>(fst) {}
 
   // Get a copy of this VectorFst. See Fst<>::Copy() for further doc.
-  virtual VectorFst<A> *Copy(bool safe = false) const {
-    return new VectorFst<A>(*this);
+  virtual VectorFst<A, S> *Copy(bool safe = false) const {
+    return new VectorFst<A, S>(*this);
   }
 
-  VectorFst<A> &operator=(const VectorFst<A> &fst) {
+  VectorFst<A, S> &operator=(const VectorFst<A, S> &fst) {
     SetImpl(fst.GetImpl(), false);
     return *this;
   }
 
-  virtual VectorFst<A> &operator=(const Fst<A> &fst) {
+  virtual VectorFst<A, S> &operator=(const Fst<A> &fst) {
     if (this != &fst) SetImpl(new Impl(fst));
     return *this;
   }
 
   // Read a VectorFst from an input stream; return NULL on error
-  static VectorFst<A> *Read(istream &strm, const FstReadOptions &opts) {
+  static VectorFst<A, S> *Read(istream &strm, const FstReadOptions &opts) {
     Impl* impl = Impl::Read(strm, opts);
-    return impl ? new VectorFst<A>(impl) : 0;
+    return impl ? new VectorFst<A, S>(impl) : 0;
   }
 
   // Read a VectorFst from a file; return NULL on error
   // Empty filename reads from standard input
-  static VectorFst<A> *Read(const string &filename) {
+  static VectorFst<A, S> *Read(const string &filename) {
     Impl* impl = ImplToExpandedFst<Impl, MutableFst<A> >::Read(filename);
-    return impl ? new VectorFst<A>(impl) : 0;
+    return impl ? new VectorFst<A, S>(impl) : 0;
   }
 
+  virtual bool Write(ostream &strm, const FstWriteOptions &opts) const {
+    return WriteFst(*this, strm, opts);
+  }
+
+  virtual bool Write(const string &filename) const {
+    return Fst<A>::WriteFile(filename);
+  }
+
+  template <class F>
+  static bool WriteFst(const F &fst, ostream &strm,
+                       const FstWriteOptions &opts);
+
   void ReserveStates(StateId n) {
     MutateCheck();
     GetImpl()->ReserveStates(n);
@@ -507,12 +582,12 @@ class VectorFst : public ImplToMutableFst< VectorFstImpl<A> > {
 // Specialization for VectorFst; see generic version in fst.h
 // for sample usage (but use the VectorFst type!). This version
 // should inline.
-template <class A>
-class StateIterator< VectorFst<A> > {
+template <class A, class S>
+class StateIterator< VectorFst<A, S> > {
  public:
   typedef typename A::StateId StateId;
 
-  explicit StateIterator(const VectorFst<A> &fst)
+  explicit StateIterator(const VectorFst<A, S> &fst)
       : nstates_(fst.GetImpl()->NumStates()), s_(0) {}
 
   bool Done() const { return s_ >= nstates_; }
@@ -530,18 +605,73 @@ class StateIterator< VectorFst<A> > {
   DISALLOW_COPY_AND_ASSIGN(StateIterator);
 };
 
+// Writes Fst to file, will call CountStates so may involve two passes if
+// called from an Fst that is not derived from Expanded.
+template <class A, class S>
+template <class F>
+bool VectorFst<A, S>::WriteFst(const F &fst, ostream &strm,
+                            const FstWriteOptions &opts) {
+  static const int kFileVersion = 2;
+  bool update_header = true;
+  FstHeader hdr;
+  hdr.SetStart(fst.Start());
+  hdr.SetNumStates(kNoStateId);
+  size_t start_offset = 0;
+  if (fst.Properties(kExpanded, false) || (start_offset = strm.tellp()) != -1) {
+    hdr.SetNumStates(CountStates(fst));
+    update_header = false;
+  }
+  uint64 properties = fst.Properties(kCopyProperties, false) |
+      Impl::kStaticProperties;
+  FstImpl<A>::WriteFstHeader(fst, strm, opts, kFileVersion, "vector",
+                             properties, &hdr);
+  StateId num_states = 0;
+  for (StateIterator<F> siter(fst); !siter.Done(); siter.Next()) {
+    typename A::StateId s = siter.Value();
+    fst.Final(s).Write(strm);
+    int64 narcs = fst.NumArcs(s);
+    WriteType(strm, narcs);
+    for (ArcIterator<F> aiter(fst, s); !aiter.Done(); aiter.Next()) {
+      const A &arc = aiter.Value();
+      WriteType(strm, arc.ilabel);
+      WriteType(strm, arc.olabel);
+      arc.weight.Write(strm);
+      WriteType(strm, arc.nextstate);
+    }
+    num_states++;
+  }
+  strm.flush();
+  if (!strm) {
+    LOG(ERROR) << "VectorFst::Write: write failed: " << opts.source;
+    return false;
+  }
+  if (update_header) {
+    hdr.SetNumStates(num_states);
+    return FstImpl<A>::UpdateFstHeader(fst, strm, opts, kFileVersion, "vector",
+                                       properties, &hdr, start_offset);
+  } else {
+    if (num_states != hdr.NumStates()) {
+      LOG(ERROR) << "Inconsistent number of states observed during write";
+      return false;
+    }
+  }
+  return true;
+}
+
 // Specialization for VectorFst; see generic version in fst.h
 // for sample usage (but use the VectorFst type!). This version
 // should inline.
-template <class A>
-class ArcIterator< VectorFst<A> > {
+template <class A, class S>
+class ArcIterator< VectorFst<A, S> > {
  public:
   typedef typename A::StateId StateId;
 
-  ArcIterator(const VectorFst<A> &fst, StateId s)
-      : arcs_(fst.GetImpl()->GetState(s)->arcs), i_(0) {}
+  ArcIterator(const VectorFst<A, S> &fst, StateId s)
+      : arcs_(fst.GetImpl()->GetState(s)->Arcs()),
+        narcs_(fst.GetImpl()->GetState(s)->NumArcs()),
+        i_(0) {}
 
-  bool Done() const { return i_ >= arcs_.size(); }
+  bool Done() const { return i_ >= narcs_; }
 
   const A& Value() const { return arcs_[i_]; }
 
@@ -560,7 +690,8 @@ class ArcIterator< VectorFst<A> > {
   void SetFlags(uint32 f, uint32 m) {}
 
  private:
-  const vector<A>& arcs_;
+  const A *arcs_;
+  size_t narcs_;
   size_t i_;
 
   DISALLOW_COPY_AND_ASSIGN(ArcIterator);
@@ -569,22 +700,23 @@ class ArcIterator< VectorFst<A> > {
 // Specialization for VectorFst; see generic version in fst.h
 // for sample usage (but use the VectorFst type!). This version
 // should inline.
-template <class A>
-class MutableArcIterator< VectorFst<A> >
+template <class A, class S>
+class MutableArcIterator< VectorFst<A, S> >
   : public MutableArcIteratorBase<A> {
  public:
+  typedef S State;
   typedef typename A::StateId StateId;
   typedef typename A::Weight Weight;
 
-  MutableArcIterator(VectorFst<A> *fst, StateId s) : i_(0) {
+  MutableArcIterator(VectorFst<A, S> *fst, StateId s) : i_(0) {
     fst->MutateCheck();
     state_ = fst->GetImpl()->GetState(s);
     properties_ = &fst->GetImpl()->properties_;
   }
 
-  bool Done() const { return i_ >= state_->arcs.size(); }
+  bool Done() const { return i_ >= state_->NumArcs(); }
 
-  const A& Value() const { return state_->arcs[i_]; }
+  const A& Value() const { return state_->GetArc(i_); }
 
   void Next() { ++i_; }
 
@@ -595,28 +727,27 @@ class MutableArcIterator< VectorFst<A> >
   void Seek(size_t a) { i_ = a; }
 
   void SetValue(const A &arc) {
-    A& oarc = state_->arcs[i_];
+    const A& oarc = state_->GetArc(i_);
     if (oarc.ilabel != oarc.olabel)
       *properties_ &= ~kNotAcceptor;
     if (oarc.ilabel == 0) {
-      --state_->niepsilons;
       *properties_ &= ~kIEpsilons;
       if (oarc.olabel == 0)
         *properties_ &= ~kEpsilons;
     }
     if (oarc.olabel == 0) {
-      --state_->noepsilons;
       *properties_ &= ~kOEpsilons;
     }
     if (oarc.weight != Weight::Zero() && oarc.weight != Weight::One())
       *properties_ &= ~kWeighted;
-    oarc = arc;
+
+    state_->SetArc(arc, i_);
+
     if (arc.ilabel != arc.olabel) {
       *properties_ |= kNotAcceptor;
       *properties_ &= ~kAcceptor;
     }
     if (arc.ilabel == 0) {
-      ++state_->niepsilons;
       *properties_ |= kIEpsilons;
       *properties_ &= ~kNoIEpsilons;
       if (arc.olabel == 0) {
@@ -625,7 +756,6 @@ class MutableArcIterator< VectorFst<A> >
       }
     }
     if (arc.olabel == 0) {
-      ++state_->noepsilons;
       *properties_ |= kOEpsilons;
       *properties_ &= ~kNoOEpsilons;
     }
@@ -644,7 +774,6 @@ class MutableArcIterator< VectorFst<A> >
 
   void SetFlags(uint32 f, uint32 m) {}
 
-
  private:
   // This allows base-class virtual access to non-virtual derived-
   // class members of the same name. It makes the derived class more
@@ -659,7 +788,7 @@ class MutableArcIterator< VectorFst<A> >
   uint32 Flags_() const { return Flags(); }
   void SetFlags_(uint32 f, uint32 m) { SetFlags(f, m); }
 
-  struct VectorState<A> *state_;
+  State *state_;
   uint64 *properties_;
   size_t i_;
 
@@ -667,10 +796,10 @@ class MutableArcIterator< VectorFst<A> >
 };
 
 // Provide information needed for the generic mutable arc iterator
-template <class A> inline
-void VectorFst<A>::InitMutableArcIterator(
+template <class A, class S> inline
+void VectorFst<A, S>::InitMutableArcIterator(
     StateId s, MutableArcIteratorData<A> *data) {
-  data->base = new MutableArcIterator< VectorFst<A> >(this, s);
+  data->base = new MutableArcIterator< VectorFst<A, S> >(this, s);
 }
 
 // A useful alias when using StdArc.
