@@ -21,10 +21,10 @@
 namespace fst {
 namespace script {
 
-bool Equal(const FstClass &fst1, const FstClass &fst2, float kDelta) {
+bool Equal(const FstClass &fst1, const FstClass &fst2, float delta) {
   if (!ArcTypesMatch(fst1, fst2, "Equal")) return false;
 
-  EqualInnerArgs args(fst1, fst2, kDelta);
+  EqualInnerArgs args(fst1, fst2, delta);
   EqualArgs args_with_retval(args);
 
   Apply<Operation<EqualArgs> >("Equal", fst1.ArcType(), &args_with_retval);
