@@ -27,27 +27,32 @@ using std::pair; using std::make_pair;
 namespace fst {
 namespace script {
 
-void ReadPotentials(const string &weight_type,
+// Reads vector of weights; returns true on success.
+bool ReadPotentials(const string &weight_type,
                     const string& filename,
                     vector<WeightClass>* potential) {
   ifstream strm(filename.c_str());
-  if (!strm)
-    LOG(FATAL) << "ReadPotentials: Can't open file: " << filename;
+  if (!strm.good()) {
+    LOG(ERROR) << "ReadPotentials: Can't open file: " << filename;
+    return false;
+  }
 
   const int kLineLen = 8096;
   char line[kLineLen];
   size_t nline = 0;
 
   potential->clear();
-  while (strm.getline(line, kLineLen)) {
+  while (!strm.getline(line, kLineLen).fail()) {
     ++nline;
     vector<char *> col;
     SplitToVector(line, "\n\t ", &col, true);
     if (col.size() == 0 || col[0][0] == '\0')  // empty line
       continue;
-    if (col.size() != 2)
-      LOG(FATAL) << "ReadPotentials: Bad number of columns, "
+    if (col.size() != 2) {
+      LOG(ERROR) << "ReadPotentials: Bad number of columns, "
                  << "file = " << filename << ", line = " << nline;
+      return false;
+    }
 
     ssize_t s = StrToInt64(col[0], filename, nline, false);
     WeightClass weight(weight_type, col[1]);
@@ -56,26 +61,33 @@ void ReadPotentials(const string &weight_type,
       potential->push_back(WeightClass::Zero());
     (*potential)[s] = weight;
   }
+  return true;
 }
 
-void WritePotentials(const string& filename,
+// Writes vector of weights; returns true on success.
+bool WritePotentials(const string& filename,
                      const vector<WeightClass>& potential) {
-  ostream *strm = &std::cout;
+  ostream *strm = &cout;
   if (!filename.empty()) {
     strm = new ofstream(filename.c_str());
-    if (!*strm)
-      LOG(FATAL) << "WritePotentials: Can't open file: " << filename;
+    if (!strm->good()) {
+      LOG(ERROR) << "WritePotentials: Can't open file: " << filename;
+      delete strm;
+      return false;
+    }
   }
 
   strm->precision(9);
   for (ssize_t s = 0; s < potential.size(); ++s)
     *strm << s << "\t" << potential[s] << "\n";
 
-  if (!*strm)
-    LOG(FATAL) << "WritePotentials: Write failed: "
+  if (strm->fail())
+    LOG(ERROR) << "WritePotentials: Write failed: "
                << (filename.empty() ? "standard output" : filename);
-  if (strm != &std::cout)
+  bool ret = !strm->fail();
+  if (strm != &cout)
     delete strm;
+  return ret;
 }
 
 
